	list p=16f876
;**************************************************************
;*
;* Pinbelegung
;*	----------------------------------	
;*	PORTA: 	0 analoger Eingang
;*		1 -
;*		2 -
;*		3 -
;*		4 -
;*	PORTB:	0 DAC	   
;*		1 DAC	    
;*		2 DAC	    
;*		3 DAC	     
;*		4 DAC	    
;*		5 DAC	    
;*		6 DAC	    
;*		7 DAC	    
;*	PORTC:	-	    
;*
;**************************************************************
;
; sprut (zero) Bredendiek 04/2010
;
; ADC-DAC-Lernbeispiel:
;
; 16F876 wandelt die an RA0 anliegende Spannung in einen
; 10-Bit-Wert um
; die 10 Bit des Wertes steuern die Frequenz eines Sinusses
; der mit einem DAC am PORTB ausgegeben wird
;
; Prozessortakt:  20 MHz
; Eingangsspannung: 0V ... 5V (Vss ... Vdd)
;
;****************************************************************************************
;
; im Quelltext wird mit dezimalen Zahlen gearbeitet
	list r=dec              ;Set the radix as decimal.
;
;****************************************************************************************
; Includedatei fr den 16F876 einbinden
	#include <P16f876.INC>	

;****************************************************************************************
; Configuration festlegen:
; Power on Timer, kein Watchdog, HS-Oscillator, kein Brown out, kein LV-programming
	__CONFIG	_PWRTE_ON & _WDT_OFF & _HS_OSC & _BODEN_OFF & _LVP_OFF

; clk = 20 MHz
; cycl = 5 MHz
;****************************************************************************************
; Variablen 

	cblock	0x20
	akku:2			;16 bit accumulator
	f1:2			;16 bit frequenz
	endc

;****************************************************************************************
; Startadresse
	org	0

start
	; RA0 wird analoger input
	; RB  wird output fuer DAC
	bsf	STATUS,RP0	; Bank 1
	movlw	B'11111111'
	movwf	TRISA
	clrf	TRISB		; RB0..7 output
	bcf	STATUS,RP0	; Bank 0

	clrf	INTCON		; interrupts disabled

	clrf	akku
	clrf	akku+1		; Phase = 0/65536 * 360grd

	movlw	1
	movwf	f1
	clrf	f1+1		; Frequenz:	f1 = 1


; ADC vorbereiten
	; ADC einschalten
	BSF	ADCON0, 0	; ADON=1
	; ADC-Eingang AN0 auswhlen
	BCF	ADCON0, 5	; ADCHS2=0
	BCF	ADCON0, 4	; ADCHS1=0
	BCF	ADCON0, 3	; ADCHS0=0
	; ADC speed fr 20 MHz einstellen
	BSF	ADCON0, 7	; ADCS1=1
	BCF	ADCON0, 6	; ADCS0=0
	; Daten rechtsbndig
	BSF	STATUS,RP0	; Bank1
	BSF	ADCON1, 7	; ADFM=1
	BCF	STATUS,RP0	; Bank0
	BSF	ADCON0, 2	; ADC starten
	goto	sine


;****************************************************************************************
; Erzeugung der Wellenform
;****************************************************************************************
; Sinus
; frequenz: f1
; benutzt Sinustabelle aus 256 Werten fuer eine Welle
; ca 24 zyklen pro loop
; f1 =    1 -> 24*256*256 zyklen -> 3,2 Hz 
; f1 = 1023 -> 24* 64     zyklen -> 3200 Hz
sine
	movlw	3
	movwf	PCLATH		; vorbereiten fuer Sin-Tabelle im Bereich 0x300..0x3ff

_next
	movf	f1,w
	addwf	akku,f
	movf	f1+1,w
	btfsc	STATUS,C
	incfsz	f1+1,w
	addwf	akku+1,f	; akku = akku + f1  (16 Bit)

	movf	akku+1,w
	call	getsine		; w = sin(w) (Page 3) ; ca. 6 cyc
	movwf	PORTB		; -> DAC

	BTFSC	ADCON0, 2	; ist der ADC fertig?
	GOTO	_del		; nein, weiter warten
	movfw	ADRESH		; obere 2 Bit auslesen
	movwf	f1+1		; obere 2 Bit nach f1+1
	BSF	STATUS,RP0	; Bank1
	movfw	ADRESL		; untere 8 Bit auslesen
	BCF	STATUS,RP0	; Bank0
	movwf	f1		; untere 8 Bit nach f1
	BSF	ADCON0, 2	; ADC starten
	goto	_next

_del
	nop
	nop
	nop
	nop
	nop
	nop
	goto	_next
	

;****************************************************************************************
; Sinustabelle aus 256 Werten
; Wert =  (sin(w/256*2*Pi) + 1) *127.5 ; fuer w = 0 ...255
; PCLATCH = 0x03
; dadurch landet addwf im Bereich 3xxh
	org	2ffh
getsine	addwf	PCL,f
sintab
	DT	126, 129, 132, 135, 138, 141, 144, 148		; Anfang der positiven Halbwelle
	DT	151, 154, 157, 160, 163, 166, 168, 171
	DT	174, 177, 180, 183, 185, 188, 191, 193
	DT	196, 199, 201, 204, 206, 208, 211, 213
	DT	215, 217, 219, 221, 223, 225, 227, 229
	DT	231, 232, 234, 236, 237, 239, 240, 241
	DT	242, 244, 245, 246, 247, 247, 248, 249
	DT	250, 250, 251, 251, 251, 252, 252, 252
	DT	252, 252, 252, 252, 251, 251, 251, 250
	DT	250, 249, 248, 247, 247, 246, 245, 244
	DT	242, 241, 240, 239, 237, 236, 234, 232
	DT	231, 229, 227, 225, 223, 221, 219, 217
	DT	215, 213, 211, 208, 206, 204, 201, 199
	DT	196, 193, 191, 188, 185, 183, 180, 177
	DT	174, 171, 168, 166, 163, 160, 157, 154
	DT	151, 148, 144, 141, 138, 135, 132, 129		; Ende der positiven Halbwelle
	DT	126, 123, 120, 117, 114, 111, 108, 104		; Anfang der negativen Halbwelle
	DT	101,  98,  95,  92,  89,  86,  84,  81
	DT	 78,  75,  72,  69,  67,  64,  61,  59
	DT	 56,  53,  51,  48,  46,  44,  41,  39
	DT	 37,  35,  33,  31,  29,  27,  25,  23
	DT	 21,  20,  18,  16,  15,  13,  12,  11
	DT	 10,   8,   7,   6,   5,   5,   4,   3
	DT	  2,   2,   1,   1,   1,   0,   0,   0
	DT	  0,   0,   0,   0,   1,   1,   1,   2
	DT	  2,   3,   4,   5,   5,   6,   7,   8
	DT	 10,  11,  12,  13,  15,  16,  18,  20
	DT	 21,  23,  25,  27,  29,  31,  33,  35
	DT	 37,  39,  41,  44,  46,  48,  51,  53
	DT	 56,  59,  61,  64,  67,  69,  72,  75
	DT	 78,  81,  84,  86,  89,  92,  95,  98
	DT	101, 104, 108, 111, 114, 117, 120, 123		; Ende der negativen Halbwelle

	end
	
;****************************************************************************************
   
